@@ -96,6 +96,8 @@ intersphinx_mapping = dict(
     sklearn=("https://scikit-learn.org/stable/", None),
     cugraph=("https://docs.rapids.ai/api/cugraph/stable/", None),
     scipy=("https://docs.scipy.org/doc/scipy/reference/", None),
+    cellrank=("https://cellrank.readthedocs.io/en/stable/", None),
+    seaborn=("https://seaborn.pydata.org/", None),
 )
 
 templates_path = ["_templates"]
@@ -119,10 +121,13 @@ from urllib.request import urlretrieve
 
 notebooks_url = "https://github.com/LouisFaure/scFates_notebooks/raw/main/"
 notebooks = [
-    "Basic_pseudotime_analysis.ipynb",
+    "Basic_Curved_trajectory_analysis.ipynb",
+    "Basic_Tree_analysis.ipynb",
     "Advanced_bifurcation_analysis.ipynb",
+    "Tree_operations.ipynb",
     "Conversion_from_CellRank_pipeline.ipynb",
     "Critical_Transition.ipynb",
+    "Beyond_scRNAseq.ipynb",
 ]
 for nb in notebooks:
     try:
@@ -139,7 +144,7 @@ html_theme_options = {
     "logo_only": True,
     # Toc options
     "collapse_navigation": True,
-    "sticky_navigation": True,
+    "sticky_navigation": False,
     "navigation_depth": 4,
     "includehidden": True,
     "titles_only": False,
