@@ -9,3 +9,5 @@ from .test_fork import test_fork
 from .module_inclusion import module_inclusion
 from .critical_transition import critical_transition
 from .linearity_deviation import linearity_deviation
+from .dendrogram import dendrogram
+from .binned_pseudotime_meta import binned_pseudotime_meta
