@@ -16,6 +16,8 @@ def test_pipeline():
     scf.tl.tree(adata, Nodes=10, use_rep="pca", method="epg", device="cpu", seed=1)
     F_PC1_epgt_cpu = adata.uns["graph"]["F"][0, :5]
 
+    scf.tl.circle(adata, Nodes=10, use_rep="pca", device="cpu", seed=1)
+
     # scf.tl.tree(adata,Nodes=10,use_rep="pca",method="epg",device="gpu",seed=1)
     # F_PC1_epgt_gpu = adata.uns['graph']['F'][0,:5]
 
@@ -52,12 +54,19 @@ def test_pipeline():
     adata_2 = scf.tl.roots(
         adata, roots=[80, 25], meeting=adata.uns["graph"]["forks"][0], copy=True
     )
+    scf.tl.root(adata, "n_counts")
+    scf.tl.root(adata, "Phox2a", tips_only=True, min_val=True)
     scf.tl.root(adata, 80)
     pp_info_time = adata.uns["graph"]["pp_info"]["time"][:5].values
 
     scf.tl.pseudotime(adata_2, n_map=2)
     scf.tl.pseudotime(adata_2)
     scf.tl.pseudotime(adata)
+    scf.tl.dendrogram(adata)
+
+    scf.pl.binned_pseudotime_meta(adata, "leiden", show_colorbar=True)
+
+    adata_ext = scf.tl.extend_tips(adata, copy=True)
 
     obs_t = adata.obs.t[:5].values
 
@@ -73,6 +82,15 @@ def test_pipeline():
     scf.pl.trajectory_3d(adata)
     scf.pl.trajectory_3d(adata, color="seg")
 
+    adata_s1 = scf.tl.subset_tree(
+        adata, root_milestone="29", milestones=["19"], mode="substract", copy=True
+    )
+    adata_s2 = scf.tl.subset_tree(
+        adata, root_milestone="29", milestones=["19"], mode="extract", copy=True
+    )
+    adata_at = scf.tl.attach_tree(adata_s1, adata_s2)
+    adata_at = scf.tl.attach_tree(adata_s1, adata_s2, linkage=("25", "19"))
+
     scf.tl.test_association(adata, n_jobs=2)
     scf.tl.test_association(adata, A_cut=0.3, reapply_filters=True)
     scf.tl.test_association(adata_2, layer="scaled", A_cut=0.3)
@@ -127,6 +145,10 @@ def test_pipeline():
         show=False,
     )
 
+    scf.pl.dendrogram(
+        adata, show_info=False, color="t", root_milestone="80", milestones=["25", "19"]
+    )
+
     scf.tl.test_fork(
         adata, layer="scaled", root_milestone="80", milestones=["25", "19"], n_jobs=2
     )
@@ -160,6 +182,10 @@ def test_pipeline():
     scf.pl.modules(adata, root_milestone="80", milestones=["25", "19"])
     scf.pl.modules(adata, root_milestone="80", milestones=["25", "19"], show_traj=True)
 
+    scf.pl.single_trend(
+        adata, root_milestone="80", milestones=["25", "19"], module="early", branch="25"
+    )
+
     scf.tl.slide_cells(adata, root_milestone="80", milestones=["25", "19"], win=200)
     cell_freq_sum = adata.uns["80->25<>19"]["cell_freq"][0].sum()
 
