@@ -1,6 +1,7 @@
-from .graph_tools import tree, curve, cleanup, getpath
+from .graph_operations import cleanup, getpath, subset_tree, attach_tree, extend_tips
+from .graph_fitting import tree, curve, circle
 from .root import root, roots
-from .pseudotime import pseudotime, refine_pseudotime, rename_milestones
+from .pseudotime import pseudotime, rename_milestones
 from .test_association import test_association
 from .fit import fit
 from .cluster import cluster
@@ -16,3 +17,4 @@ from .bifurcation_tools import test_fork, branch_specific, activation, activatio
 # from .activation_tools import activation, activation_lm
 from .conversion import cellrank_to_tree
 from .linearity_deviation import linearity_deviation
+from .dendrogram import dendrogram
