@@ -4,7 +4,7 @@ os.environ["OPENBLAS_NUM_THREADS"] = "1"
 os.environ["OMP_NUM_THREADS"] = "1"
 os.environ["MKL_NUM_THREADS"] = "1"
 
-from typing import Union, Optional, Tuple, Collection, Sequence, Iterable
+from typing import Union, Optional, Iterable
 
 import numpy as np
 import pandas as pd
@@ -190,7 +190,8 @@ def test_fork(
         topleave = fork_stat[m].iloc[:, :-1].idxmax(axis=1).apply(lambda mil: dct[mil])
 
         ## test for upregulation
-        logg.info("    test for upregulation for each leave vs root")
+        if n_map == 1:
+            logg.info("    test for upregulation for each leave vs root")
         leaves_stat = list()
         for leave in leaves:
             subtree = getpath(img, root, graph["tips"], leave, graph, df).sort_values(
@@ -793,7 +794,7 @@ def activation_lm(
         },
         index=df.index,
     )
-    df[["slope", "pval", "fdr", "prefork_signi"]] = df_res[
+    df[["slope", "pre_pval", "pre_fdr", "prefork_signi"]] = df_res[
         ["slope", "pval", "fdr", "prefork_signi"]
     ]
     df["module"] = "late"
