@@ -4,7 +4,7 @@ os.environ["OPENBLAS_NUM_THREADS"] = "1"
 os.environ["OMP_NUM_THREADS"] = "1"
 os.environ["MKL_NUM_THREADS"] = "1"
 
-from typing import Union, Optional, Tuple, Collection, Sequence, Iterable
+from typing import Union, Optional, Iterable
 
 import numpy as np
 import pandas as pd
@@ -71,7 +71,7 @@ def test_association(
     n_jobs
         number of cpu processes used to perform the test.
     spline_df
-        dimension of the basis used to represent the smooth term.
+        dimension of the basis used to represent the smooth term (has no effect for now).
     fdr_cut
         FDR (Benjamini-Hochberg adjustment) cutoff on significance; significance if FDR < fdr_cut.
     A_cut
